@@ -25,15 +25,16 @@ __authors__ = [
     'Thomas Chiroux', ]
 
 
+import os
 import glob
 import datetime
 import difflib
 
 
 # dependencies imports
-from bottle import request, response, template, abort, redirect
+from bottle import request, response, template, abort, redirect, static_file
 import docutils
-from docutils.core import publish_string, publish_parts
+from docutils.core import publish_parts, publish_doctree
 from docutils.writers.html4css1 import Writer as HisWriter
 from docutils import io, nodes
 
@@ -45,6 +46,7 @@ from attowiki.git_tools import (check_repo, commit,
                                 add_file_to_repo,
                                 commit_history,
                                 read_committed_file)
+from attowiki.pdf import produce_pdf
 
 
 def view_meta_cheat_sheet():
@@ -89,7 +91,7 @@ def view_meta_admonition(admonition_name, name=None):
 
         :admonition_name: (str) -- name of the admonition
     """
-    print "meta admo: %s - %s" % (admonition_name, name)
+    print("meta admo: %s - %s" % (admonition_name, name))
     admonition = None
 
     if admonition_name == 'todo':
@@ -101,21 +103,21 @@ def view_meta_admonition(admonition_name, name=None):
     else:
         return abort(404)
 
-    doc2_content=""
+    doc2_content = ""
 
     doc2_output, doc2_pub = docutils.core.publish_programmatically(
-                                source_class=io.StringInput,
-                                source=doc2_content,
-                                source_path=None,
-                                destination_class=io.StringOutput,
-                                destination=None, destination_path=None,
-                                reader=None, reader_name='standalone',
-                                parser=None, parser_name='restructuredtext',
-                                writer=HisWriter(), writer_name=None,
-                                settings=None, settings_spec=None,
-                                settings_overrides=None,
-                                config_section=None,
-                                enable_exit_status=False)
+        source_class=io.StringInput,
+        source=doc2_content,
+        source_path=None,
+        destination_class=io.StringOutput,
+        destination=None, destination_path=None,
+        reader=None, reader_name='standalone',
+        parser=None, parser_name='restructuredtext',
+        writer=HisWriter(), writer_name=None,
+        settings=None, settings_spec=None,
+        settings_overrides=None,
+        config_section=None,
+        enable_exit_status=False)
 
     section1 = nodes.section("{0}_list_file".format(admonition_name))
     doc2_pub.reader.document.append(section1)
@@ -123,7 +125,8 @@ def view_meta_admonition(admonition_name, name=None):
                          "{0} LIST".format(admonition_name.upper()))
     doc2_pub.reader.document.append(title1)
     if name is None:
-        rst_files = [filename[2:-4] for filename in sorted(glob.glob("./*.rst"))]
+        rst_files = [filename[2:-4] for filename in sorted(
+            glob.glob("./*.rst"))]
         rst_files.reverse()
     else:
         rst_files = [filename[2:-4] for filename in
@@ -162,10 +165,10 @@ def view_meta_admonition(admonition_name, name=None):
                 paragraph = nodes.paragraph()
                 file_target = nodes.target(ids=[file],
                                            names=[file],
-                                           refuri="/"+file)
+                                           refuri="/" + file)
                 file_ref = nodes.reference(file, file,
                                            name=file,
-                                           refuri="/"+file)
+                                           refuri="/" + file)
                 paragraph.append(nodes.Text("in "))
                 paragraph.append(file_ref)
                 paragraph.append(file_target)
@@ -215,7 +218,7 @@ def view_cancel_edit(name=None):
         files = glob.glob("{0}.rst".format(name))
         if len(files) > 0:
             reset_to_last_commit()
-            return redirect('/'+name)
+            return redirect('/' + name)
         else:
             return abort(404)
 
@@ -263,6 +266,37 @@ def view_edit(name=None):
             return abort(404)
 
 
+def view_pdf(name=None):
+    """render a pdf file based on the given page
+
+    .. note:: this is a bottle view
+
+    Keyword Arguments:
+        :name: (str) -- name of the rest file (without the .rst extension)
+                        MANDATORY
+    """
+    if name is None:
+        return view_meta_index()
+
+    files = glob.glob("{0}.rst".format(name))
+    if len(files) > 0:
+        file_handle = open(files[0], 'r')
+        dest_filename = name + '.pdf'
+        doctree = publish_doctree(file_handle.read())
+        try:
+            produce_pdf(doctree_content=doctree,
+                        filename=dest_filename)
+        except:
+            raise
+            #return redirect(name)
+        else:
+            return static_file(dest_filename,
+                               root='',
+                               download=True)
+    else:
+        return redirect(name)
+
+
 def view_page(name=None):
     """serve a page name
 
@@ -315,9 +349,9 @@ def view_page(name=None):
     if len(files) > 0:
         file_handle = open(files[0], 'r')
         html_body = publish_parts(file_handle.read(),
-                                   writer=HisWriter(),
-                                   settings=None,
-                                   settings_overrides=None)['html_body']
+                                  writer=HisWriter(),
+                                  settings=None,
+                                  settings_overrides=None)['html_body']
         history = commit_history("{0}.rst".format(name))
         return template('page',
                         type="view",
@@ -328,7 +362,8 @@ def view_page(name=None):
                         gitref=None,
                         content=html_body)
     else:
-        return abort(404)
+        return static_file(name, '')
+
 
 def view_history(name, gitref):
     """serve a page name from git repo (an old version of a page)
